"""Computes the gyral coordinate system.

For each point in space defines the radial, sulcal, and gyral
orientations
"""
import nibabel
import numpy as np
from fsl.wrappers import LOAD, gps
from nibabel import gifti
from numpy import linalg

from mcot.surface import grid, utils
from mcot.surface.cortical_mesh import CorticalMesh


class WeightedOrientation(object):
    """Can compute an averaged radial/tangential fiber orientations for every
    voxel."""
    _flip_inpr = False

    def __init__(self, white, pial, sulcal_depth, resolution_grid, target_affine):
        """Prepares computation of the radial/tangential fiber orientations.

        :param white: white/gray matter boundary
        :type white: CorticalMesh
        :param pial: pial surface
        :type pial: CorticalMesh
        :param sulcal_depth: sulcal depth
        :param resolution_grid: voxel size of the accelerator grid in mm
        :param target_affine: (4x4) array giving the voxel -> mm conversion for the target grid
        """
        self.white = white
        self.pial = pial
        self.sulcal_depth = sulcal_depth
        self.resolution_grid = resolution_grid
        self.target_affine = target_affine

        shape, affine = grid.bounding(pial, resolution_grid)
        self.affine = affine

        self.white_hit = grid.intersect(white, shape, affine)
        self.pial_hit = grid.intersect(pial, shape, affine)

        self.white_vox = self.white.apply_affine(target_affine)
        self.pial_vox = self.pial.apply_affine(target_affine)

        self.white_grad = self.white_vox.gradient(sulcal_depth)
        self.pial_grad = self.pial_vox.gradient(sulcal_depth)
        self.white_normal = self.white_vox.normal()
        self.pial_normal = self.pial_vox.normal()

        self.white_grad_point = self.white_vox.gradient(sulcal_depth, atpoint=True)
        self.pial_grad_point = self.pial_vox.gradient(sulcal_depth, atpoint=True)
        self.white_normal_point = self.white_vox.normal(atpoint=True)
        self.pial_normal_point= self.pial_vox.normal(atpoint=True)

        self.smooth_orient = np.zeros((0, 3))

    def average_line_vox(self, mm_index, norient=1000, power_dist=-1.):
        """Computes the radial/tangential hemisphere at the given point.

        This uses the main FOTACS algorithm:

        1. Draw straight lines through the point of interest connecting the cortical surfaces at both sides
        2. Linearly interpolate the normal/sulcal depth gradient along this line

        Repeat these steps for `norient` random orientations.
        Average these orientations with the weighting set by the line length ** `power_dist`.

        :param mm_index: (3, ) vector of position in mm
        :param norient: number of random orientations to try
        :param power_dist: power-law used to downweight longer faces (`weight = dist ** power_dist`)
        :return: Tuple with 4 elements:

            1. interpolated normal
            2. interpolated sulcal depth gradient
            3. length of shortest line hitting surface on both sides
            4. number between 0 and 0.5 indicating location along shortest line (0 if at edge, 0.5 if in middle of gyrus)
        """
        if self.smooth_orient.shape[0] != norient:
            rand_orient = np.random.randn(3, norient)
            rand_orient /= np.sqrt(np.sum(rand_orient ** 2, 0))

            self.smooth_orient = gps(LOAD, ndir=3000)

        orientations = np.concatenate((self.smooth_orient, -self.smooth_orient), 0)
        w_ix, w_pos = self.white_hit.ray_intersect(mm_index, orientations)
        normal_inpr = np.sum(self.white.normal()[:, w_ix] * orientations.T, 0)
        segment = 1 if normal_inpr[w_ix != -1].sum() < 0 else 2
        if segment == 1:
            o_ix, o_pos = self.pial_hit.ray_intersect(mm_index, -orientations, pos_inpr=1)
            use = (w_ix != -1) & (o_ix != -1) & (normal_inpr <= 0)
        else:
            o_ix, o_pos = w_ix[norient:], w_pos[norient:]
            w_ix, w_pos = w_ix[:norient], w_pos[:norient]
            use = (w_ix != -1) & (o_ix != -1) & (normal_inpr[:norient] >= 0) & (normal_inpr[norient:] >= 0)

        other_grad = self.white_grad if segment == 2 else self.pial_grad
        other_normal = -self.white_normal if segment == 2 else self.pial_normal

        # linear interpolation of the orientations
        dist_white = np.sqrt(np.sum((w_pos[use, :] - mm_index) ** 2, -1))[:, None]
        dist_other = np.sqrt(np.sum((o_pos[use, :] - mm_index) ** 2, -1))[:, None]
        dist = dist_white + dist_other
        weight = dist ** power_dist

        res = []
        for other_inp, white_inp in [(other_normal, self.white_normal),
                                     (other_grad, self.white_grad)]:
            linear_interp = (dist_white * other_inp.T[o_ix[use], :] + dist_other * white_inp.T[w_ix[use], :]) / dist
            linear_interp *= weight / np.sqrt(np.sum(linear_interp ** 2, -1))[:, None]
            linear_interp[~np.isfinite(linear_interp)] = 0.
            cov = np.dot(linear_interp.T, linear_interp)
            val, vec = linalg.eigh(cov)
            res.append(vec[:, np.argmax(val)])
        res.append(np.inf if dist.size == 0 else dist.min())
        ratio_length = np.nan
        if dist.size != 0:
            idx = np.argmin(dist)
            ratio_length = min((dist_white[idx], dist_other[idx])) / dist[idx]
        res.append(ratio_length)
        return tuple(res)

    def average_line_grid(self, shape, zval=None, norient=1000, power_dist=-1.):
        """Computes the radial/tangential orientations on a grid.

        This uses the main FOTACS algorithm:
        1. Draw straight lines through the point of interest connecting the cortical surfaces at both sides
        2. Linearly interpolate the normal/sulcal depth gradient along this line
        Repeat these steps for `norient` random orientations.
        Average these orientations with the weighting set by the line length ** `power_dist`.

        :param shape: (nx, ny, nz); defines the shape of the output volume
        :param zval: only evaluates a single horizontal slice if set
        :param norient: number of random orientations to try
        :param power_dist: power-law used to downweight longer faces (`weight = dist ** power_dist`)
        """
        if len(shape) != 3:
            use = shape != 0
            shape = use.shape
        else:
            use = np.ones(shape, dtype='bool')
        res = np.zeros(shape + (3, 2), dtype='f4')
        thickness = np.zeros(shape + (2, ), dtype='f4')
        thickness[()] = np.nan
        use = use & (grid.signed_distance(self.pial, shape, affine=self.target_affine) < 0)
        if zval is not None:
            use = use & (np.arange(shape[-1])[None, None, :] == zval)
        res[~use, :] = np.nan
        for ixvox in zip(*np.where(use)):
            av_line = self.average_line_vox(utils.affine_mult(self.target_affine, ixvox),
                                            norient=norient, power_dist=power_dist)
            thickness[ixvox] = av_line[2:]
            res[ixvox] = np.array(av_line[:2]).T
        return res, thickness

    @staticmethod
    def _weights(surface_vertices, target, normal, pos_inpr=0, power_dist=-1.):
        """Computes the weight of every surface element on every
        volume_element.

        :param surface_vertices: (N, 3) array with vertex locations along the surface
        :param target: (3, ) array with position of interest
        :param normal: (N, 3) array with the surface normals at the vertices
        :param pos_inpr: ignore vertex element with normals pointing to the point of interest (if pos_inpr < 0) or away from the point of interest (if pos_inpr < 0); default: no filtering
        :param power_dist: resulting weights = distance ** power_dist
        :return: (N, ) array of vertex weights (set to 0 for ignored vertices; see `pos_inpr` parameter)
        """
        offset = surface_vertices - np.array(target)[:, None]
        weight = np.sum(offset ** 2, 0) ** (power_dist / 2)
        if pos_inpr != 0:
            inpr = np.sum(offset * normal, 0)
            if pos_inpr > 0:
                weight[inpr > 0] = 0
            else:
                weight[inpr < 0] = 0
        return weight

    def average_point(self, voxel, power_dist=-1.):
        """Computes the radial and tangential hemisphere at given voxel.

        This algorithm averages the normal/sulcal depth gradient of every vertex weighted by its distance from the point of interest

        :param voxel: (3, ) array with point of interest in voxel coordinates
        :param power_dist: power-law used to downweight longer faces (`weight = dist ** power_dist`)
        """
        res = np.zeros((3, 2))
        cov_radial = np.zeros((3, 3))
        cov_tangential = np.zeros((3, 3))
        for surf, norm, grad, pos_inpr in [(self.white_vox, self.white_normal_point, self.white_grad_point, 0),
                         (self.pial_vox, self.pial_normal_point, self.pial_grad_point, 1 if self._flip_inpr else -1)]:
            weight = self._weights(surf.vertices, voxel, norm, power_dist=power_dist, pos_inpr=pos_inpr)
            weighted_norm = weight[None, :] * norm
            cov_radial += np.dot(weighted_norm, weighted_norm.T)

            weighted_grad = weight[None, :] * grad
            cov_tangential += np.dot(weighted_grad, weighted_grad.T)
        val, vec = linalg.eigh(cov_radial)
        res[:, 0] = vec[:, np.argmax(val)]
        val, vec = linalg.eigh(cov_tangential)
        res[:, 1] = vec[:, np.argmax(val)]
        return res

    def average_point_grid(self, shape, zval=None, power_dist=-1., outside_pial=False):
        """Computes the primary orientations at every point within the pial
        surface.

        This algorithm averages the normal/sulcal depth gradient of every vertex weighted by its distance from the point of interest

        :param shape: shape of the resulting array
        :param zval: only process a single horizontal slice
        :param power_dist: power-law used to downweight longer faces (`weight = dist ** power_dist`)
        :param outside_pial: if True also run for voxels outside of the pial surface
        """
        if len(shape) != 3:
            use = shape != 0
            shape = use.shape
        else:
            use = np.ones(shape, dtype='bool')
        if not outside_pial:
            use = use & (grid.signed_distance(self.pial, shape, self.target_affine) < 0)
        if zval is not None:
            use = use & (np.arange(shape[-1])[None, None, :] == zval)
        res = np.zeros(use.shape + (3, 2))
        for voxel in zip(*np.where(use)):
            res[voxel] = self.average_point(voxel, power_dist=power_dist)
        return res

    def closest_vertex_grid(self, shape, zval=None, outside_pial=False):
        """Computes the primary orientations at every point within the pial
        surface.

        This algorithm selects the normal/sulcal depth gradient from the closest point.

        :param shape: shape of the resulting array
        :param zval: only process a single horizontal slice
        :param outside_pial: if True also run for voxels outside of the pial surface
        """
        if len(shape) != 3:
            use = shape != 0
            shape = use.shape
        else:
            use = np.ones(shape, dtype='bool')
        dist_pial = grid.signed_distance(self.pial, shape, self.target_affine)
        dist_white = grid.signed_distance(self.white, shape, self.target_affine)

        if not outside_pial:
            use = use & (dist_pial < 0)
        if zval is not None:
            use = use & (np.arange(shape[-1])[None, None, :] == zval)
        res = np.zeros(use.shape + (3, 2))

        idx_get_pial = grid.closest_surface(self.pial, use & (abs(dist_pial) < abs(dist_white)), self.target_affine)
        res[idx_get_pial != -1, :, 0] = self.pial_normal_point.T[idx_get_pial[idx_get_pial != -1]]
        res[idx_get_pial != -1, :, 1] = self.pial_grad_point.T[idx_get_pial[idx_get_pial != -1]]
        idx_get_white = grid.closest_surface(self.white, use & (abs(dist_white) <= abs(dist_pial)), self.target_affine)
        res[idx_get_white != -1, :, 0] = self.white_normal_point.T[idx_get_white[idx_get_white != -1]]
        res[idx_get_white != -1, :, 1] = self.white_grad_point.T[idx_get_white[idx_get_white != -1]]
        return res / np.sqrt(np.sum(res ** 2, -2)[..., None, :])


def align_vector_field(from_field, to_field):
    """Returns a new vector field parallel to `from_field`, which is aligned
    with `to_field`

    :param from_field: (Nx, Ny, Nz, 3, Nf) original vector field
    :param to_field: (Nx, Ny, Nz, 3, Nf) reference vector field with the correct rough hemisphere
    :return: same as `from_field`, but with every vector flipped that has a negative inner product with `to_field`
    """
    res = from_field.copy()
    _, flip = np.broadcast_arrays(res, (np.sum(from_field * to_field, -2) < 0)[:, :, :, None, :])
    res[flip] *= -1
    return res


def make_perpendicular(radial, tangential):
    """Projects the tangential field, so that it is perpendicular to the radial
    field.

    :param radial: (Nx, Ny, Nz, 3) array with the radial orientations
    :param tangential: (Nx, Ny, Nz, 3) array with the tangential orientations
    :return: (Nx, Ny, Nz, 3, 3) array with the radial orientations [..., 0],
        the projected tangential orientations [..., 1] and the orientations perpendicular to both [..., 2].
        All output orientations will be normalized.
    """
    proj_tang = tangential - np.sum(radial * tangential, -1)[..., None] * radial
    new_tang = np.cross(radial, proj_tang)
    return np.stack([arr / np.sqrt(np.sum(arr ** 2, -1))[..., None] for arr in [radial, proj_tang, new_tang]], -1)


def run_from_args(args):
    if args.thickness is not None and args.algorithm != 'line':
        raise ValueError("Optional thickness output only available for the 'line algorithm, not for %s" % args.algorithm)
    white = CorticalMesh.read(args.white)
    pial = CorticalMesh.read(args.pial)
    if args.sulcal_depth is None:
        sd = np.zeros(white.nvertices)
    else:
        sd = gifti.read(args.sulcal_depth).darrays[0].data
    img_mask = nibabel.load(args.mask)
    mask = img_mask.get_data()
    if mask.ndim != 3:
        raise ValueError("Input mask should be 3-dimenasional")

    wo = WeightedOrientation(white, pial, sd, 1., img_mask.affine)
    wo._flip_inpr = args.flip_inpr
    rough = wo.closest_vertex_grid(mask, zval=args.zval, outside_pial=args.outside_pial)
    if args.algorithm == 'closest':
        field = rough
    elif args.algorithm == 'line':
        field, thickness = wo.average_line_grid(mask, norient=args.norient, power_dist=args.power_dist, zval=args.zval)
        # line algorithm might fail for voxels on the edge
        replace = ~np.isfinite(thickness[..., 0])
        field[replace] = rough[replace]
        if args.thickness is not None:
            nibabel.Nifti1Image(thickness, affine=img_mask.affine).to_filename(args.thickness)
    elif args.algorithm == 'interp':
        field = wo.average_point_grid(mask, power_dist=args.power_dist, zval=args.zval, outside_pial=args.outside_pial)
    flipped = align_vector_field(field, rough)
    coords = make_perpendicular(flipped[..., 0], flipped[..., 1])

    nibabel.Nifti1Image(coords, affine=img_mask.affine).to_filename(args.output)



"""Script that fits a sigmoid to the radial index across the white/gray matter
boundary."""

import numpy as np

try:
    import pymc3
    from theano import sparse, tensor
except ImportError:
    raise ImportError("Fitting the radial transition model requires pymc3")
from nibabel import cifti2
import nibabel
from scipy import optimize
from scipy import sparse as sp_sparse

from mcot.surface.all import CorticalMesh, grid


def model(radial_index, white, affine=None, surf_mask=None, min_dist=-2., smooth_weight=0.7,
          watson=False, idx_vertex=None, dist=None):
    """Creates a probabilistic model of the transition across the white/gray
    matter bounary.

    :param radial_index: (Nx, Ny, Nz) array with the radial index
    :param white: white/gray matter boundary
    :param affine: (4, 4) array with the transformation from voxel to mm space
    :param surf_mask: (Nvertex, ) boolean array, which is True on vertices to be included in the fit
    :param min_dist: only include voxels within this distance from WM/GM boundary
    :param smooth_weight: weighting for the smoothing parameter
    :param watson: assume a Watson distribution for the radial index rather than a normal distribution
    :param idx_vertex: (Nx, Ny, Nz) array with index of closest vertex
    :param dist: distance from the WM/GM boundary
    :return: pymc3 model which fits the sigmoidal transition across the surface
    """
    if affine is None:
        affine = np.eye(4)

    if surf_mask is None:
        surf_mask = np.ones(white.nvertex, dtype='bool')

    vol_mask = np.isfinite(radial_index) & (radial_index != 0.)
    if dist is None:
        dist = grid.signed_distance(white, radial_index.shape, affine)
    if idx_vertex is None:
        idx_vertex_raw = grid.closest_surface(white, vol_mask, affine)
    else:
        idx_vertex_raw = idx_vertex

    nvertex = surf_mask.sum()
    idx_real_vertex = -np.ones(surf_mask.size, dtype='i4')
    idx_real_vertex[surf_mask] = np.arange(nvertex)
    voxel_vertex = idx_real_vertex[idx_vertex_raw]
    voxel_vertex[idx_vertex_raw == -1] = -1
    voxel_use = (voxel_vertex != -1) & (dist > min_dist) & vol_mask & (dist != 0)

    gpp = white.graph_point_point()[surf_mask, :][:, surf_mask]
    smooth = sp_sparse.diags(np.array(1 / gpp.sum(-1)).ravel()).dot(gpp)

    assert abs(smooth.dot(np.ones(smooth.shape[0])) - 1).max() < 1e-13

    # radial index model
    with pymc3.Model() as model:
        d0 = pymc3.Flat('d0', testval=0.5, shape=nvertex)
        log_sigma = pymc3.Flat('log_sigma', testval=-0.5, shape=nvertex)
        sigma = tensor.exp(log_sigma)
        # radial index is zero in WM, 1 in GM
        model_ri = 1 / (1 + tensor.exp(-(dist[voxel_use] - d0[voxel_vertex[voxel_use]]) / sigma[voxel_vertex[voxel_use]]))
        if watson:
            pymc3.Potential('alignment', model_ri * abs(radial_index[voxel_use]) +
                            tensor.sqrt(1 - model_ri ** 2) * np.sqrt(1 - radial_index[voxel_use] ** 2))
        else:
            pymc3.Potential('alignment', -(model_ri - abs(radial_index[voxel_use])) ** 2)
        d_neigh = sparse.structured_dot(smooth, tensor.stack([d0], -1))[:, 0]
        pymc3.Potential('d0_smooth', -smooth_weight * (d_neigh - d0) ** 2)
        ls_neigh = sparse.structured_dot(smooth, tensor.stack([log_sigma], -1))[:, 0]
        pymc3.Potential('ls_smooth', -smooth_weight * (ls_neigh - log_sigma) ** 2)

        # additional output to check
        pymc3.Deterministic('model_ri_1d', model_ri)
        for name, arr_1d in [('model_ri', model_ri), ('observed_ri', abs(radial_index[voxel_use])),
                             ('dist', dist[voxel_use]), ('d0', d0[voxel_vertex[voxel_use]]),
                             ('sigma', sigma[voxel_vertex[voxel_use]])]:
            vol_nan = tensor.fill(radial_index, np.nan)
            vol_filled = tensor.set_subtensor(vol_nan[voxel_use.nonzero()], arr_1d)
            pymc3.Deterministic('%s_3d' % name, vol_filled)
    return model


def run_from_args(args):
    img = nibabel.load(args.coord)
    coord = img.get_data()
    white = CorticalMesh.read(args.white)
    dti = nibabel.load(args.dyad).get_data()
    gdti = (coord * dti[..., None]).sum(-2)
    if args.mask is None:
        surf_mask = np.ones(white.nvertices, dtype='bool')
    else:
        surf_mask = nibabel.load(args.mask).darrays[0].data != 0
    idx_vertex = None if args.idx_vertex is None else nibabel.load(args.idx_vertex).get_data()
    if idx_vertex.ndim == 4:
        idx_vertex = idx_vertex[..., 0]
    distance = None if args.distance is None else nibabel.load(args.distance).get_data()
    tofit = model(abs(gdti[..., 0]), white, img.affine, surf_mask, min_dist=args.min_dist, smooth_weight=args.weight,
                  watson=args.watson, idx_vertex=idx_vertex, dist=distance)
    res = pymc3.find_MAP(model=tofit, fmin=optimize.fmin_cg)

    if args.radial_index is not None:
        model_ri = tofit.fastfn(tofit.model_ri_3d)(res)
        obs_ri = tofit.fastfn(tofit.observed_ri_3d)(res)
        nibabel.Nifti1Image(np.stack([model_ri, obs_ri], -1), img.affine).to_filename(args.radial_index)

    bm = cifti2.BrainModelAxis.from_surface(np.where(surf_mask)[0], surf_mask.size, white.anatomy.cifti)
    mat = np.stack([res['d0'], res['log_sigma'], np.exp(res['log_sigma'])])
    cifti2.Cifti2Image(mat, (cifti2.ScalarAxis(['offset', 'log_width', 'width']), bm)).to_filename(args.output)



#!/usr/bin/env python
"""Extract radial, sulcal, and gyral orientations from gyral coordinate NIFTI
file."""


def main():
    import argparse

    parser = argparse.ArgumentParser("Extract radial, sulcal, and gyral dyads from a coord NIFTI file")
    parser.add_argument('coord', help='name of the coord file')
    parser.add_argument('-b', '--base', help='Basename of the output files')
    parser.add_argument('-r', '--radial', help='Filename for the radial output (overrides the --base option)')
    parser.add_argument('-s', '--sulcal', help='Filename for the sulcal output (overrides the --base option)')
    parser.add_argument('-g', '--gyral', help='Filename for the gyral output (overrides the --base option)')

    args = parser.parse_args()
    from mcot.surface import utils
    utils.gcoord_split(args)



#!/usr/bin/env python
"""Converts dyads into gyral coordinate system."""


def main():
    import argparse

    parser = argparse.ArgumentParser("Converts the dyads from the cardinal to the gyral coordinate system")
    parser.add_argument('-d', '--dyads', required=True, help='NIFTI file with the estimated fibre orientations')
    parser.add_argument('-c', '--coord', required=True, nargs='+', help='NIFTI file with the gyral coordinates produced by gcoord_gen')
    parser.add_argument('-o', '--output', required=True, help='Output NIFTI files with the fibre orientations in gyral coordinates')

    args = parser.parse_args()
    from mcot.surface import utils
    utils.gcoord_mult(args)



#!/usr/bin/env python
"""Extract fibre orientation transition boundary."""


def main():
    import argparse
    parser = argparse.ArgumentParser('Fits a sigmoidal model to the radial index across the white/gray matter boundary')
    parser.add_argument('coord', help='NIFTI file with radial & tangential orientations from gcood_gen')
    parser.add_argument('dyad', help='NIFTI file with DTI or other hemisphere')
    parser.add_argument('white', help='GIFTI file with white/gray matter boundary')
    parser.add_argument('output', help='CIFTI dscalar file with the output parameters')
    parser.add_argument('-m', '--mask', help='GIFTI file with non-zero values for the vertices to use')
    parser.add_argument('-w', '--weight', type=float, default=1., help='Weighting of the smoothing term (defaults: 1., which works well for HCP data)')
    parser.add_argument('-md', '--min-dist', type=float, default=-3, help='Minimal voxel distance from the white/gray matter boundary (default: -3)')
    parser.add_argument('--watson', action='store_true', help='Assumes Watson rather than Normal distribution for radial index')
    parser.add_argument('--idx-vertex', help='Optional NIFTI file with the index of the vertex that every voxel should be assigned to')
    parser.add_argument('--distance', help='Optional NIFTI file with the distance from the WM/GM boundary')
    parser.add_argument('-ri', '--radial-index', help='Optional NIFTI file output containing the radial index of the best-fit model')
    args = parser.parse_args()
    from mcot.gcoord import radial_transition
    radial_transition.run_from_args(args)



#!/usr/bin/env python
"""Generates the gyral coordinates in a volume."""


def main():
    import argparse
    parser = argparse.ArgumentParser("Computes the radial and tangential hemisphere for every point in space", formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument("algorithm", choices=['line', 'interp', 'closest'], help="""Select which algorithm to use to compute the radial/tangential orientations. There are three options:
1. 'line': strongly recommended with -pd -1: The surface normal (& sulcal depth gradient) are linearly interpolated along N randomly oriented lines and averaged
2. 'interp': best with -pd -3: All surface normals (& sulcal depth depth gradients) are averaged with the weighting depending on the distance from the voxel of interest
3. 'closest': Use the normal (& sulcal depth gradient) from the closest vertex""")
    parser.add_argument("output", help='Output NIFTI file where the gyral coordinate system will be stored')
    parser.add_argument("-m", "--mask", required=True, help="computes the gyral coordinates for any non-zero voxels")
    parser.add_argument("-w", "--white", required=True,
                        help="GIFTI file defining the white/gray matter boundary (ending with .surf.gii)")
    parser.add_argument("-p", "--pial", required=True,
                        help="GIFTI file defining the pial surface (ending with .surf.gii)")
    parser.add_argument("-sd", "--sulcal_depth",
                        help="GIFTI file defining the sulcal depth (ending with .shape.gii or .func.gii)")
    parser.add_argument("-N", "--norient", default=300, type=int,
                        help="Number of random line orientations to average over (only if algorithm='line'); " +
                             "default: 300")
    parser.add_argument("--outside_pial", action='store_true', help='Include voxels outside of the pial surface (does not work for the line algorithm)')
    parser.add_argument("-pd", "--power_dist", default=-1., type=float,
                        help="weights in averaging = d ** `power_dist`, where weights is the line length " +
                             "(if algorithm='line') or the distance between vertex and point of interest " +
                             "(if algorithm='interp'); default: -1")
    parser.add_argument("-t", "--thickness",
                        help="Optional NIFTI output file containing the length of the shortest line connecting " +
                             "the cortical surfaces through the voxel and the voxel location along this line " +
                             "(0 for edge, 0.5 for gyral centre). Only valid for the `line` algorithm")
    parser.add_argument("--flip-inpr", action='store_true', help=argparse.SUPPRESS)
    parser.add_argument("--zval", default=None, type=int, help=argparse.SUPPRESS)
    args = parser.parse_args()
    if args.outside_pial and args.algorithm == 'line':
        raise ValueError("Algorithm 'line' can not include voxels outside of the pial surface.")
    from mcot.gcoord import orientation
    orientation.run_from_args(args)



#!/usr/bin/env python.app
"""GUI for interacting with gyral coordinate scripts."""
try:
    from gooey import Gooey, GooeyParser
except ModuleNotFoundError:
    raise ImportError("You need to install the gooey python library to run the gyral coordinate GUI")


@Gooey
def main():
    parser = GooeyParser(description="Utilities to work with the gyral coordinate system")
    subs = parser.add_subparsers(help='commands', dest='commands')

    generator = subs.add_parser(
            name='generate', description="Computes the radial and tangential hemisphere for every point in space")
    generator.add_argument("algorithm", choices=['line', 'interp', 'closest'], help="""Select which algorithm to use to compute the radial/tangential orientations. There are three options:
1. 'line': strongly recommended with -pd -1: The surface normal (& sulcal depth gradient) are linearly interpolated along N randomly oriented lines and averaged
2. 'interp': best with -pd -3: All surface normals (& sulcal depth depth gradients) are averaged with the weighting depending on the distance from the voxel of interest
3. 'closest': Use the normal (& sulcal depth gradient) from the closest vertex""")
    generator.add_argument("output", help='Output NIFTI file where the gyral coordinate system will be stored',
                           widget='FileSaver')
    generator.add_argument("-m", "--mask", help="computes the gyral coordinates for any non-zero voxels",
                           widget='FileChooser')
    generator.add_argument("-w", "--white", required=True,
                           help="GIFTI file defining the white/gray matter boundary (ending with .surf.gii)",
                           widget='FileChooser')
    generator.add_argument("-p", "--pial", required=True,
                           help="GIFTI file defining the pial surface (ending with .surf.gii)", widget='FileChooser')
    generator.add_argument("-sd", "--sulcal_depth",
                           help="GIFTI file defining the sulcal depth (ending with .shape.gii or .func.gii)",
                           widget='FileChooser')
    generator.add_argument(
            "-N", "--norient", default=300, type=int,
            help="Number of random line orientations to average over (only if algorithm='line'); default: 1000")
    generator.add_argument(
            "-pd", "--power_dist", default=-1., type=float,
            help="weights in averaging = d ** `power_dist`, where weights is the line length (if algorithm='line') or the distance between vertex and point of interest (if algorithm='interp'); default: -1")
    generator.add_argument(
            "-t", "--thickness", widget='FileChooser',
            help="Optional NIFTI output file containing the length of the shortest line connecting the cortical surfaces through the voxel (only valid for the `line` algorithm")

    splitter = subs.add_parser(name='split',
                               description="Extract radial, sulcal, and gyral dyads from a coord NIFTI file")
    splitter.add_argument('coord', help='name of the coord file', widget='FileChooser')
    splitter.add_argument('-b', '--base', help='Basename of the output files', widget='FileSaver')
    splitter.add_argument('-r', '--radial', help='Filename for the radial output (overrides the --base option)',
                          widget='FileSaver')
    splitter.add_argument('-s', '--sulcal', help='Filename for the sulcal output (overrides the --base option)',
                          widget='FileSaver')
    splitter.add_argument('-g', '--gyral', help='Filename for the gyral output (overrides the --base option)',
                          widget='FileSaver')

    mult = subs.add_parser(description="Converts the dyads from the cardinal to the gyral coordinate system",
                           name='multiply')
    mult.add_argument('-d', '--dyads', required=True, help='NIFTI file with the estimated fibre orientations',
                      widget='FileChooser')
    mult.add_argument('-c', '--coord', required=True, nargs='+',
                      help='NIFTI file with the gyral coordinates produced by gcoord_gen', widget='MultiFileChooser')
    mult.add_argument('-o', '--output', required=True,
                      help='Output NIFTI files with the fibre orientations in gyral coordinates', widget='FileSaver')

    transition = subs.add_parser(
            description='Fits a sigmoidal model to the radial index across the white/gray matter boundary',
            name='transition')
    transition.add_argument('coord', help='NIFTI file with radial & tangential orientations from gcood_gen',
                            widget='FileChooser')
    transition.add_argument('dyad', help='NIFTI file with DTI or other hemisphere', widget='FileChooser')
    transition.add_argument('white', help='GIFTI file with white/gray matter boundary', widget='FileChooser')
    transition.add_argument('output', help='CIFTI dscalar file with the output parameters', widget='FileSaver')
    transition.add_argument('-m', '--mask', help='GIFTI file with non-zero values for the vertices to use',
                            widget='FileChooser')
    transition.add_argument('-w', '--weight', type=float, default=1.,
                            help='Weighting of the smoothing term (defaults: 1., which works well for HCP data)')
    transition.add_argument('-md', '--min-dist', type=float, default=-3,
                            help='Minimal voxel distance from the white/gray matter boundary (default: -3)')
    transition.add_argument('--watson', action='store_true',
                            help='Assumes Watson rather than Normal distribution for radial index')
    transition.add_argument(
            '--idx-vertex', widget='FileChooser',
            help='Optional NIFTI file with the index of the vertex that every voxel should be assigned to')
    transition.add_argument('--distance', help='Optional NIFTI file with the distance from the WM/GM boundary',
                        widget='FileChooser')
    transition.add_argument('-ri', '--radial-index', widget='FileSaver',
                            help='Optional NIFTI file output containing the radial index of the best-fit model')

    args = parser.parse_args()

    if args.commands == 'generate':
        args.flip_inpr = False
        args.zval = None
        from mcot.gcoord import orientation
        orientation.run_from_args(args)
    elif args.commands == 'split':
        from mcot.surface import utils
        utils.gcoord_split(args)
    elif args.commands == 'multiply':
        from mcot.surface import utils
        utils.gcoord_mult(args)
    elif args.commands == 'transition':
        from mcot.gcoord import radial_transition
        radial_transition.run_from_args(args)
    else:
        raise ValueError("Unrecognized sub-command %s" % args.commands)
